import random
import sys
import time
import lxml.html as lh
import requests
import pandas as pd
import collections
import warnings

from typing import Union


def check_valid_ip(string: str) -> bool:
    valid = False
    if "." in string:
        elements_array = string.strip().split(".")
        if len(elements_array) == 4:
            for i in elements_array:
                if i.isnumeric() and int(i) >= 0 and int(i) <= 255:
                    valid = True
                else:
                    valid = False
                    break
    return valid


dict_str = {
    'yes': True,
    '': False,
    'no': False,
}


def convert_str_to_bool(string: str) -> bool:
    return dict_str.get(string)


list_columns_to_clean = ['google', 'https']


def clean_dataframe(df: pd.DataFrame):
    for column in list_columns_to_clean:
        if column in df.columns:
            df[column] = df[column].apply(lambda x: convert_str_to_bool(x))
    return df


class FreeProxy:
    def __init__(self, country: Union[str, collections.abc.Iterable] = None,
                 country_code: Union[str, collections.abc.Iterable] = None,
                 timeout: float = 0.5, anonym: Union[str, bool] = None, rand: bool = True,
                 https: Union[bool, str] = True,
                 prefered_country: Union[str, collections.abc.Iterable] = None,
                 prefered_country_code: Union[str, collections.abc.Iterable] = None,
                 refresh_after: int = 900, google: bool = False, verbose: bool = False):
        self.country = country
        self.country_code = country_code
        self.prefered_country = prefered_country
        self.prefered_country_code = prefered_country_code
        self.timeout = timeout
        self.anonym = anonym
        self.https = https
        self.random = rand
        self.proxies = None
        self.fetched_at = None
        self.refresh_after = refresh_after
        self.filtered_df = None
        self.google = google
        self.verbose = verbose

    def verbose_print(self, message):
        if self.verbose:
            print(message)

    def series_to_proxy(self, series) -> dict:
        http_str = 'https' if self.https else 'http'
        proxy_set = {
            http_str: f"{series.name}:{series['port']}"
        }
        return proxy_set, series.name

    def get_proxy_list(self):
        try:
            page = requests.get('https://www.sslproxies.org')
            doc = lh.fromstring(page.content)
            tr_elements = doc.xpath('//*[@id="list"]//tr')
            list_proxies = []
            for tr_element in tr_elements:
                if check_valid_ip(tr_element[0].text_content()) and tr_element[0].text_content() is not None:
                    dict_tmp = {}
                    for counter, attribute in enumerate(
                            ['ip_address', 'port', 'country_code', 'country', 'anonymity', 'google', 'https',
                             'last_checked']):
                        dict_tmp[attribute] = tr_element[counter].text_content()
                    list_proxies.append(dict_tmp)
            self.proxies = pd.DataFrame(list_proxies)
            self.proxies = self.proxies.set_index('ip_address')
            self.proxies = clean_dataframe(self.proxies)
            self.fetched_at = time.time()
        except requests.exceptions.RequestException as e:
            print(e)
            sys.exit(1)

    @staticmethod
    def get_filter_str(country: Union[str, collections.abc.Iterable],
                       country_code: Union[str, collections.abc.Iterable],
                       anonymity: Union[str, collections.abc.Iterable], https: bool, google:bool) -> str:
        args = locals()
        filter_str = ""
        for arg in args:
            if arg == "https" and args[arg] == 'any':
                continue
            if args[arg] is not None:
                if isinstance(args[arg], bool):
                    filter_str += f"(self.proxies['{arg}'] == {args[arg]})&"
                if isinstance(args[arg], str):
                    filter_str += f"(self.proxies['{arg}'] == '{args[arg]}') & "
                if isinstance(args[arg], collections.abc.Iterable) and not isinstance(args[arg],(bytes, str)):
                    filter_str += f"(self.proxies['{arg}'].isin({args[arg]})) &"
        if filter_str == "":
            return ""
        filter_str = filter_str[: len(filter_str) - 3] + filter_str[len(filter_str) - 3:].replace("&", "")
        return f"self.proxies[{filter_str}]"

    def find_working_proxy(self) -> dict:
        if self.filtered_df.empty:
            return None
        if self.random:
            for i in range(len(self.filtered_df)):
                random_proxy = self.filtered_df.loc[random.choice(list(self.filtered_df.index))]
                random_proxy, ipaddress = self.series_to_proxy(random_proxy)
                proxy = self.check_if_proxy_is_working(random_proxy)
                if proxy:
                    return proxy
                else:
                    self.filtered_df.drop(ipaddress, inplace=True)
        else:
            for proxy in self.filtered_df.index:
                proxy_inner = self.filtered_df.loc[proxy]
                proxy_inner, ipaddress = self.series_to_proxy(proxy_inner)
                proxy_inner = self.check_if_proxy_is_working(proxy_inner)
                if proxy_inner:
                    return proxy_inner

    def check_if_proxy_is_working(self, proxy) -> dict:
        http_str = 'https' if self.https else 'http'
        try:
            with requests.get(f'{http_str}://www.google.com', proxies=proxy, timeout=self.timeout, stream=True) as r:
                if r.raw.connection.sock:
                    if r.raw.connection.sock.getpeername()[0] == proxy[http_str].split(':')[0]:
                        return proxy
        except Exception:
            return False

    def get(self) -> dict:
        
        if self.proxies is None:
            self.get_proxy_list()
        elif time.time() - self.fetched_at >= self.refresh_after:
            self.get_proxy_list()

        if self.prefered_country is not None or self.prefered_country_code is not None:
            filter_str = self.get_filter_str(self.prefered_country, self.prefered_country_code, self.anonym, self.https, self.google)
        else:
            filter_str = self.get_filter_str(self.country, self.country_code, self.anonym, self.https, self.google)

        if filter_str != "":
            exec(f"self.filtered_df = {filter_str}.copy()")
        else:
            self.filtered_df = self.proxies

        working_proxy = self.find_working_proxy()
        if working_proxy is not None:
            return working_proxy

        warnings.warn("For the prefered country no working proxies have been found. Checking Countries. If countries is None all countries will be checked")
        if self.prefered_country is not None or self.prefered_country_code is not None:
            filter_str = self.get_filter_str(self.country, self.country_code, self.anonym, self.https, self.google)
            if filter_str != "":
                exec(f"self.filtered_df = {filter_str}.copy()")
            else:
                self.filtered_df = self.proxies
            working_proxy = self.find_working_proxy()
            if working_proxy is not None:
                return working_proxy




import requests
import pandas as pd
import urllib
import collections
import json
import lxml.html as lh

from requests.exceptions import ConnectionError
from aisexplorer.Exceptions import NotSupportedParameterTypeError, NotSupportedParameterError, NoResultsError, \
    CloudflareError
from aisexplorer.Proxy import FreeProxy
from aisexplorer.Filters import Filters
from tenacity import retry, stop_after_attempt, wait_fixed


def before_retry(retry_state):
    object = retry_state.args[0]
    if object.proxy:
        object.verbose_print(retry_state.outcome.result)
        object.verbose_print("Renewing Proxy and trying again in 15 seconds")
        object.renew_proxy()
    else:
        object.verbose_print("Retrying again in 15 seconds")


def error_callback(retry_state):
    raise NoResultsError(f"After {str(retry_state.attempt_number)} Attempts still no results are given. "
                         f"If you think this is an Error in the module raise an Issue at "
                         f"https://github.com/reyemb/AISExplorer ")


class AIS:
    retry_options = {
        "stop": stop_after_attempt(10),
        "wait": wait_fixed(15),
        "after": before_retry,
        "retry_error_callback": error_callback
    }
    def __init__(self, proxy=False, verbose=False, columns="all", columns_excluded=None, print_query = False,
                 num_retries=10, seconds_wait=15, filter_config={}, return_df=False, return_total_count=False,
                 **proxy_config):
        # Setting Retry Options
        if num_retries != 10:
            AIS.retry_options['stop'] = stop_after_attempt(num_retries)
        if seconds_wait != 15:
            AIS.retry_options['wait'] = wait_fixed(seconds_wait)

        self.return_df = return_df
        self.return_total_count = return_total_count
        self.verbose = verbose
        self.columns = columns
        self.columns_excluded = columns_excluded
        self.print_query = print_query
        self.session = requests.Session()
        self.set_filters(filter_config)
        self.session.headers.update({
            'user-agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:93.0) Gecko/20100101 Firefox/93.0',
            'Vessel-Image': '005bf958a6548a79c6d3a42eba493e339624',
        })
        self.set_column_url()
        if proxy:
            if proxy_config.get('proxy_config') is not None:
                self.freeproxy = FreeProxy(**proxy_config.get('proxy_config'))
            else:
                self.freeproxy = FreeProxy()
            self.verbose_print("Searching for proxy...")
            self.session.proxies = self.freeproxy.get()
            self.verbose_print("Proxy found...")
            self.proxy = proxy
            self.burned_proxies = []
        else:
            self.proxy = False

    def set_filters(self, filter_config):
        self.filters = Filters(**filter_config)

    def set_column_url(self):
        possible_columns = ["time_of_latest_position", "flag", "shipname", "photo", "recognized_next_port",
                            "reported_eta", "reported_destination", "current_port", "imo", "mmsi", "ship_type",
                            "show_on_live_map", "area", "area_local", "lat_of_latest_position",
                            "lon_of_latest_position", "fleet", "status", "eni", "speed", "course", "draught",
                            "navigational_status", "year_of_build", "length", "width", "dwt", "current_port_unlocode",
                            "current_port_country", "callsign"]
        if self.columns != "all":
            columns_selected = self.columns
        else:
            columns_selected = possible_columns

        if self.columns_excluded is not None:
            if isinstance(self.columns_excluded, str) or isinstance(self.columns_excluded, collections.abc.Iterable):
                if isinstance(self.columns_excluded, str):
                    try:
                        columns_selected.remove(self.columns_excluded)
                    except ValueError:
                        raise NotSupportedParameterError("exclude", possible_columns, self.columns_excluded)
                if isinstance(self.columns_excluded, collections.abc.Iterable) and \
                        not isinstance(self.columns_excluded, (str, bytes)):
                    for element in self.columns_excluded:
                        try:
                            possible_columns.remove(element)
                        except ValueError:
                            raise NotSupportedParameterError("exclude", possible_columns, element)
            else:
                raise NotSupportedParameterTypeError("exclude", "collections.abc.Iterable or str",
                                                     type(self.columns_excluded))

        if isinstance(columns_selected, str):
            columns_url = columns_selected
        else:
            columns_url = ",".join(columns_selected)
        self.columns_url = columns_url

    def verbose_print(self, message):
        if self.verbose:
            print(message)

    def query_print(self, message):
        if self.print_query:
            print(message)

    def check_proxy(self):
        if self.proxy:
            if not self.freeproxy.check_if_proxy_is_working(self.session.proxies):
                self.verbose_print("Proxy has to be renewed")
                self.verbose_print("Looking for new proxy...")
                self.session.proxies = self.freeproxy.get()
                self.verbose_print("Proxy found...")

    def renew_proxy(self):
        self.verbose_print("Looking for new proxy...")
        self.session.proxies = self.freeproxy.get()
        self.verbose_print("Proxy found...")

    @retry(**retry_options)
    def get_area_data(self, area):
        possible_areas = {
            "ADRIA":	"Adriatic Sea",
            "AG":	"Arabian Sea",
            "ALASKA":	"Alaska",
            "ANT":	"Antarctica",
            "BALTIC":	"Baltic Sea",
            "BSEA":	"Black Sea",
            "CARIBS":	"Caribbean Sea",
            "CASPIAN":	"Caspian Sea",
            "CCHINA":	"Central China",
            "CISPAC":	"CIS Pacific",
            "EAFR":	"East Africa",
            "EAUS":	"East Australia",
            "ECCA":	"East Coast Central America",
            "ECCAN":	"East Coast Canada",
            "ECI":	"East Coast India",
            "ECSA":	"East Coast South America",
            "EMED":	"East Mediterranean",
            "GLAKES":	"Great Lakes",
            "INDO":	"Indonesia",
            "INLSAM":	"Inland, South America",
            "INLCN":	"Inland, China",
            "INLEU":	"Inland, Europe",
            "INLRU":	"Inland, Russia",
            "INLUS":	"Inland, USA",
            "JAPAN":	"Japan Coast",
            "NAUS":	"North Australia",
            "NCCIS":	"North Coast CIS",
            "NCHINA":	"North China",
            "NCSA":	"North Coast South America",
            "NOATL":	"North Atlantic",
            "NORDIC":	"Norwegian Coast",
            "NPAC":	"North Pacific",
            "PHIL":	"Philippines",
            "RSEA":	"Red Sea",
            "SAFR":	"South Africa",
            "SCHINA":	"South China",
            "SEASIA":	"South-East Asia",
            "SIND":	"South Indian Ocean",
            "SPAC":	"South Pacific",
            "UKC":	"UK Coast & Atlantic",
            "USEC":	"US East Coast",
            "USG":	"Gulf of Mexico",
            "USWC":	"US West Coast",
            "WAFR":	"West Africa",
            "WAUS":	"West Australia",
            "WCCA":	"West Coast Central America",
            "WCCAN":	"West Coast Canada",
            "WCI":	"West Coast India",
            "WCSA":	"West South America",
            "WMED":	"West Mediterranean"
        }
        if isinstance(area, str):
            if area not in possible_areas.keys():
                raise NotSupportedParameterError("area", possible_areas.keys, area)
            areas_long = urllib.parse.quote_plus(area)
            area_short = area
        if isinstance(area, collections.abc.Iterable) and not isinstance(area, (str, bytes)):
            for element in area:
                if element not in possible_areas:
                    raise NotSupportedParameterError("area", possible_areas.keys(), element)
            areas_long = ",".join([urllib.parse.quote_plus(possible_areas[element]) for element in area])
            area_short = ",".join(area)

        request_url = f"https://www.marinetraffic.com/en/reports?asset_type=vessels&columns={self.columns_url}" \
                      f"&area_in|in|{areas_long}|area_in={area_short}{self.filters.to_query(ignore_filter='global_area')}"
        referer_url = f"https://www.marinetraffic.com/en/data/?asset_type=vessels&columns={self.columns_url}" \
                      f"&area_in|in|{areas_long}|area_in={area_short}{self.filters.to_query(ignore_filter='global_area')}"
        self.query_print("referer_url: " + referer_url)
        self.query_print("request_url: " + request_url)
        return self.return_response(request_url, referer_url)

    @retry(**retry_options)
    def get_location(self, mmsi):
        if isinstance(mmsi, int):
            mmsi = str(mmsi)
        request_url = f"https://www.marinetraffic.com/en/reports?asset_type=vessels&columns={self.columns_url}" \
                      f"&mmsi|eq|mmsi={mmsi}{self.filters.to_query(ignore_filter='mmsi')}"
        referer_url = f"https://www.marinetraffic.com/en/data/?asset_type=vessels&columns={self.columns_url}" \
                      f"&mmsi|eq|mmsi={mmsi}{self.filters.to_query(ignore_filter='mmsi')}"
        self.query_print("referer_url: " + referer_url)
        self.query_print("request_url: " + request_url)
        return self.return_response(request_url, referer_url)

    @retry(**retry_options)
    def get_data(self):
        request_url = f"https://www.marinetraffic.com/en/reports?asset_type=vessels&columns={self.columns_url}" \
                      f"{self.filters.to_query()}"
        referer_url = f"https://www.marinetraffic.com/en/data/?asset_type=vessels&columns={self.columns_url}" \
                      f"{self.filters.to_query()}"
        self.query_print("referer_url: " + referer_url)
        self.query_print("request_url: " + request_url)
        return self.return_response(request_url, referer_url)

    @retry(**retry_options)
    def get_data_by_url(self, url):
        referer_url = url
        request_url = url.replace("data", "reports")
        return self.return_response(request_url, referer_url)

    def return_response(self, request_url, referer_url):
        self.session.headers['Referer'] = referer_url
        try:
            response = self.session.get(request_url)
        except ConnectionError as ce:
            self.verbose_print('Proxy has died. Looking for new proxy...')
            raise ce
        self.check_response_cloudflare(response)
        self.verbose_print(f"Used proxy: {self.session.proxies}")
        if response.status_code == 200:
            if self.return_df:
                if self.return_total_count:
                    return pd.DataFrame(json.loads(response.text)["data"]), json.loads(response.text)["totalCount"]
                else:
                    return pd.DataFrame(json.loads(response.text)["data"])
            else:
                if self.return_total_count:
                    return json.loads(response.text)["data"], json.loads(response.text)["totalCount"]
                else:
                    return json.loads(response.text)["data"]
        else:
            return response.text

    def check_response_cloudflare(self, response):
        doc = lh.fromstring(response.content)
        titles = doc.xpath('//title')
        if titles:
            title = titles[0].text_content()
            if "Cloudflare" in title:
                self.verbose_print(f"Cloudflare has detected unusual behavior. Changing Proxy...")
                raise CloudflareError()


import warnings
from collections.abc import Iterable
from aisexplorer.Exceptions import NotSupportedKeyError, NotSupportedKeyTypeError, NotSupportedArgumentType


class StrFilter:
    dict_var = {
        'vessel_name': 'shipname',
        'destination_port': 'recognized_next_port_in',
        'reported_dest': 'reported_destinations',
        'callsign': 'callsign',
        'current_port': 'current_port_in'
    }

    def __init__(self, key, value):
        self.key = key
        if not isinstance(value, str):
            raise NotSupportedKeyTypeError(key, type(value), [str])
        self.value = value

    def to_query(self):
        if self.key == 'vessel_name':
            return f"&{self.dict_var[self.key]}|begins|{self.dict_var[self.key]}={self.value}"
        elif self.key == 'recognized_next_port_in':
            return f"&{self.dict_var[self.key]}|begins|{self.dict_var[self.key]}_name={self.value}"
        elif self.key == 'callsign':
            return f"&{self.dict_var[self.key]}|eq|{self.dict_var[self.key]}={self.value}"
        elif self.key == 'report_dest':
            return f"&{self.dict_var[self.key]}|eq|{self.dict_var[self.key]}={self.value}"
        elif self.key == 'current_port':
            return f"&{self.dict_var[self.key]}|begins||{self.dict_var[self.key]}={self.value}"
        return ""


class SliderFilter:
    dict_var = {
        'lon': 'lon_of_latest_position_between',
        'lat': 'lat_of_latest_position_between',
        'latest_report': 'time_of_latest_position_between',
        'speed': 'speed_between',
        'course': 'course_between',
        'dwt': 'dwt_between',
        'built': 'year_of_build_between',
        'length': 'length_between',
        'width': 'width_between',
        'draught': 'draught_between'
    }

    def __init__(self, key, values):
        self.key = key
        if isinstance(values, str) or not isinstance(values, Iterable):
            raise NotSupportedKeyTypeError(key, type(values), [Iterable])
        if len(values) != 2:
            raise NotSupportedArgumentType(key, 2, len(values))
        self.value = [str(value) for value in values]

    def to_query(self):
        if self.key == 'latest_report':
            return f"&{self.dict_var[self.key]}|gte|{self.dict_var[self.key]}={','.join(self.value)}"
        elif self.key == 'course':
            return f"&{self.dict_var[self.key]}|range_circle|{self.dict_var[self.key]}={','.join(self.value)}"
        else:
            return f"&{self.dict_var[self.key]}|range|{self.dict_var[self.key]}={','.join(self.value)}"


class IntFilter:
    dict_var = {
        'imo': '',
        'emi': 'emi',
        'mmsi': 'mmsi',
    }

    def __init__(self, key, value):
        if not isinstance(value, int):
            raise NotSupportedKeyTypeError(key, type(value), [int])
        self.key = key
        self.value = value

    def to_query(self):
        return f"&{self.key}|eq|{self.key}={self.value}"


class ListFilter:
    dict_var = {
        'flag': 'flag_in',
        'vessel_type': 'ship_type_in',
        'global_area': 'area_in',
        'local_area': 'area_local_in',
        'nav_status': 'navigational_status_in',
        'current_port_country': 'current_port_country_in',
    }

    def __init__(self, key, value):
        if not isinstance(value, (list, dict)):
            raise NotSupportedKeyTypeError(key, type(value), [list, dict])
        if isinstance(value, list):
            warnings.warn("List has been given by default filter will filter all elements IN the list. If you want to "
                          "to filter for elements which are not in the list use a dict instead")
            self.value = value
            self.operator = "in"
        elif isinstance(value, str):
            self.value = value
            self.operator = "in"
        else:
            self.value = value['values']
            self.operator = value['operator']
        self.key = key

    def to_query(self):
        return f"&{self.dict_var[self.key]}|{self.operator}|{self.value}|{self.dict_var[self.key]}={self.value}"


class Filters:
    possible_filters = {
        'imo': IntFilter,
        'emi': IntFilter,
        'mmsi': IntFilter,
        'vessel_name': StrFilter,
        'callsign': StrFilter,
        'current_port': StrFilter,
        'reported_dest': StrFilter,
        'destination_port': StrFilter,
        'latest_report': SliderFilter,
        'lat': SliderFilter,
        'lon': SliderFilter,
        'speed': SliderFilter,
        'course': SliderFilter,
        'dwt': SliderFilter,
        'built': SliderFilter,
        'length': SliderFilter,
        'width': SliderFilter,
        'draught': SliderFilter,
        'flag': ListFilter,
        'vessel_type': ListFilter,
        'global_area': ListFilter,
        'local_area': ListFilter,
        'nav_status': ListFilter,
        'current_port_country': ListFilter
    }

    def __init__(self, **kwargs):
        self.filters = []
        for key, value in kwargs.items():
            if key not in self.possible_filters:
                raise NotSupportedKeyError(key, list(self.possible_filters))
            self.filters.append(self.possible_filters[key](key, value))

    def to_query(self, ignore_filter=None):
        res_str = ""
        for custom_filter in self.filters:
            if ignore_filter is not None:
                if isinstance(ignore_filter, str):
                    if custom_filter.key is ignore_filter:
                        continue
                else:
                    if custom_filter.key in ignore_filter:
                        continue
            res_str += custom_filter.to_query()
        return res_str



import pandas as pd

int_cols = ['SHIP_ID', 'IMO', 'MMSI', 'TYPE_COLOR', 'TIMEZONE', 'COUNT_PHOTOS', 'NEXT_PORT_ID', 'COURSE', 'YOB', 'DWT',
            'ETA_OFFSET', 'DISTANCE_TO_GO', 'PORT_ID', 'INLAND_ENI']
float_cols = ['LON', 'LAT', 'SPEED', 'DRAUGHT', 'LENGTH', 'WIDTH']
bool_cols = ['CTA_ROUTE_FORECAST']
string_cols = ['CALLSIGN', 'SHIPNAME', 'CODE2', 'COUNTRY', 'NEXT_PORT_NAME', 'NEXT_PORT_COUNTRY', 'DESTINATION',
               'TYPE_SUMMARY', 'STATUS_NAME', 'CURRENT_PORT_UNLOCODE', 'CURRENT_PORT_COUNTRY', 'AREA_CODE', 'STATUS',
               'AREA_NAME', 'CURRENT_PORT', 'COLLECTION_NAME']
unix_cols = ['LAST_POS', 'ETA']
timestamp_cols = ['ETA_UPDATED']


def set_types_df(df):
    df = df.copy()
    for column in df.columns:
        if column in int_cols:
            df[column] = pd.to_numeric(df[column])
        elif column in float_cols:
            df[column] = pd.to_numeric(df[column])
        elif column in bool_cols:
            df[column] = df[column].astype(bool)
        elif column in string_cols:
            df[column] = df[column].astype(str)
        elif column in unix_cols:
            if column == "ETA":
                df[column] = df[column].apply(lambda x: x if x != "masked" else None)
            df[column] = pd.to_numeric(df[column])
            df[column] = pd.to_datetime(df[column], unit="s")
        elif column in timestamp_cols:
            df[column] = pd.to_datetime(df[column])
    return df



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['aisexplorer', 'aisexplorer.Utils']

package_data = \
{'': ['*']}

install_requires = \
['lxml>=4.6.4,<5.0.0',
 'pandas>=1.3.4,<2.0.0',
 'requests>=2.26.0,<3.0.0',
 'tenacity>=8.0.1,<9.0.0']

setup_kwargs = {
    'name': 'aisexplorer',
    'version': '0.0.15',
    'description': 'Wrapper to fetch data from marinetraffic',
    'long_description': '# AISExplorer\nAISExplorer can be used to locate vessels or to scrape all vessel in an specific AREA. \nAlso free proxies can be used for scraping. Proxies will be scraped from https://www.sslproxies.org/, tested and used.\nAlso Fallback are implemented\n\n## Installation\n\n```\npip install aisexplorer\n```\n\n## Usage\n\n### Find vessel by MMIS\n\n```python\nfrom aisexplorer.AIS import AIS\n\nAIS().get_location(211281610)\n```\n\n### Find vessels in Area\n\n**maximum 500 vessels**\n\n```python\nfrom aisexplorer.AIS import AIS\n\nAIS(return_df= True).get_area_data("EMED")\n```\nOutput is limited to 500 rows.\n[Areas](https://help.marinetraffic.com/hc/en-us/articles/214556408-Areas-of-the-World-How-does-MarineTraffic-segment-them-) can be found here\n\n### Get Table via URL\n\n```python\nfrom aisexplorer.AIS import AIS\n\nAIS(return_df = True).get_data_by_url("https://www.marinetraffic.com/en/data/?asset_type=vessels&columns=time_of_latest_position:desc,flag,shipname,photo,recognized_next_port,reported_eta,reported_destination,current_port,imo,ship_type,show_on_live_map,area,lat_of_latest_position,lon_of_latest_position,speed,length,width&area_in|in|West%20Mediterranean,East%20Mediterranean|area_in=WMED,EMED&time_of_latest_position_between|gte|time_of_latest_position_between=60,NaN")\n```\n\n\n### Use Proxies\n\n```python\nfrom aisexplorer.AIS import AIS\n\nAIS(return_df = True, Proxy = True).get_data_by_url("https://www.marinetraffic.com/en/data/?asset_type=vessels&columns=time_of_latest_position:desc,flag,shipname,photo,recognized_next_port,reported_eta,reported_destination,current_port,imo,ship_type,show_on_live_map,area,lat_of_latest_position,lon_of_latest_position,speed,length,width&area_in|in|West%20Mediterranean,East%20Mediterranean|area_in=WMED,EMED&time_of_latest_position_between|gte|time_of_latest_position_between=60,NaN")\n\n```\n\n\n## Next Steps\n\n- Add more potential proxy lists\n- Find a way to change the number of retrys\n- Reset new filters\n\n## Changelog\n\n### 2021-12-10\n\n- Added Fallback if proxy has died\n- Added get data by url\n- Added Check if requests was filtered by cloudflare\n\n### 2021-12-5\n\n- Added Filters early stage\n- Added Retry Options\n- Added some new exceptions\n\n### 2021-11-27\n\n- Added Proxy Option\n\n\n\n\n\n\n',
    'author': 'reyemb',
    'author_email': 'reyemb.coding@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/reyemb/AISExplorer',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



import difflib


class MalformedFunctionError(Exception):
    def __init__(self, message):
        super().__init__(f"Function is malformed\n{message}")


class NotSupportedParameterTypeError(MalformedFunctionError):
    def __init__(self, parameter, supported_types, given_type):
        super().__init__(f"{parameter} does only support the following instances {', '.join(supported_types)} "
                         f"but {given_type} has been given")


class NotSupportedParameterError(MalformedFunctionError):
    def __init__(self, parameter, supported_arguments, given_argument):
        super().__init__(f"{parameter} only accepts the following arguments {','.join(supported_arguments)}, "
                         f"but {given_argument} was given. "
                         f"Did you mean {difflib.get_close_matches(given_argument, supported_arguments)}")


class MalformedFilterError(Exception):
    def __init__(self, message):
        super().__init__(f"Filter is malformed\n{message}")


class NotSupportedKeyError(MalformedFilterError):
    def __init__(self, supported_keys, given_key):
        super().__init__(f"The following keys are accepted: {','.join(supported_keys)}, but {given_key} was given. "
                         f" you mean {difflib.get_close_matches(given_key, supported_keys)}")


class NotSupportedKeyTypeError(MalformedFilterError):
    def __init__(self, key, value, accept_types):
        super().__init__(
            f"For the {key} the following values types are accepted: {','.join(accept_types)}, but {value} was given.")


class NotSupportedArgumentType(MalformedFilterError):
    def __init__(self, key, given_form, accepted_form):
        super().__init__(
            f"For the Filter {key} the arguments should have a length of {accepted_form} but {given_form} was given.")


class NoResultsError(Exception):
    def __init__(self, message):
        super().__init__(message)


class CloudflareError(Exception):
    def __init__(self):
        super().__init__("Cloudflare detected unusual behaviour")


